@@ -21,7 +21,6 @@ from typing import Any, Awaitable, Callable, Dict, List, Optional, TypeVar, Unio
 import typing
 from datetime import timedelta
 
-import typing_extensions
 from restate.serde import DefaultSerde, Serde
 
 T = TypeVar('T')
@@ -42,7 +41,7 @@ class RunOptions(typing.Generic[T]):
     """The serialization/deserialization mechanism. - if the default serde is used, a default serializer will be used based on the type.
                     See also 'type_hint'."""
     max_attempts: Optional[int] = None
-    """The maximum number of retry attempts to complete the action.
+    """The maximum number of retry attempts, including the initial attempt, to complete the action.
                             If None, the action will be retried indefinitely, until it succeeds.
                             Otherwise, the action will be retried until the maximum number of attempts is reached and then it will raise a TerminalError."""
     max_retry_duration: Optional[timedelta] = None
@@ -245,7 +244,6 @@ class Context(abc.ABC):
         This timestamp will be stable across retries and replays.
         """
 
-    @typing_extensions.deprecated("`run` is deprecated, use `run_typed` instead for better type safety")
     @overload
     @abc.abstractmethod
     def run(self,
@@ -260,6 +258,8 @@ class Context(abc.ABC):
         """
         Runs the given action with the given name.
 
+        DEPRECATED: Use ctx.run_typed instead.
+
         Args:
             name: The name of the action.
             action: The action to run.
@@ -275,7 +275,6 @@ class Context(abc.ABC):
 
         """
 
-    @typing_extensions.deprecated("`run` is deprecated, use `run_typed` instead for better type safety")
     @overload
     @abc.abstractmethod
     def run(self,
@@ -290,6 +289,8 @@ class Context(abc.ABC):
         """
         Runs the given coroutine action with the given name.
 
+        DEPRECATED: Use ctx.run_typed instead.
+
         Args:
             name: The name of the action.
             action: The action to run.
@@ -305,7 +306,6 @@ class Context(abc.ABC):
 
         """
 
-    @typing_extensions.deprecated("`run` is deprecated, use `run_typed` instead for better type safety")
     @abc.abstractmethod
     def run(self,
             name: str,
@@ -319,6 +319,8 @@ class Context(abc.ABC):
         """
         Runs the given action with the given name.
 
+        DEPRECATED: Use ctx.run_typed instead.
+
         Args:
             name: The name of the action.
             action: The action to run.
